package view;

import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.HashMap;
import java.util.Map;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;

import controller.Annotation;

/**
 * This class represents the UI component responsible for setting the limits of words
 * that must be annotated 
 * @author gonta
 *
 */
public class LimitAnnotationView extends JDialog {
	
	
	private static final long serialVersionUID = 1L;
	
	Annotation annotation;
	
	JCheckBox checkVerb; 
	JTextField txtVerb;
	
	JCheckBox checkNoun; 
	JTextField txtNoun;
	
	JPanel noun_panel;
	JPanel verb_panel;
	
	Map<String,Double> limits = new HashMap<String,Double>();
	JButton accept_button;
	JButton cancel_button;
	
	JLabel jLabel1;
	JLabel jLabel2;
	
	/**
	 * Constructor
	 * @param frame
	 */
	public LimitAnnotationView(final JFrame frame ){
		super( frame, "Limites de Anotação" );
		
		limits.put("NOUN", 1.0);
		limits.put("VERB", 1.0);
		
		initComponents();

	}
		
	/**
	 * This method inicialize the components included in the view
	 */
	 private void initComponents() {

	        accept_button = new javax.swing.JButton();
	        cancel_button = new javax.swing.JButton();
	        verb_panel = new javax.swing.JPanel();
	        checkVerb = new javax.swing.JCheckBox();
	        jLabel1 = new javax.swing.JLabel();
	        txtVerb = new javax.swing.JTextField();
	        noun_panel = new javax.swing.JPanel();
	        checkNoun = new javax.swing.JCheckBox();
	        jLabel2 = new javax.swing.JLabel();
	        txtNoun = new javax.swing.JTextField();

	        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
	        setTitle("Limites da Anotação");
	        setResizable(false);

	        accept_button.setText("Aceitar");

			final Component component = this;
	        accept_button.addActionListener(new ActionListener() {
				@Override
				public void actionPerformed(ActionEvent e) {
					boolean res = true;
					if( checkNoun.isSelected() ) {
						limits.put("NOUN",1.0);
						txtNoun.setText("");
					} else {
						if( !txtNoun.getText().equals("") && Integer.parseInt(txtNoun.getText()) <= 100 &&
								Integer.parseInt(txtNoun.getText()) >= 0 ) {
							limits.put("NOUN", Double.parseDouble(txtNoun.getText())/100);
						} else {
							res = false;
						}							
					}
					
					if( checkVerb.isSelected() ) {
						limits.put("VERB",1.0);
						txtVerb.setText("");
					} else {
						if( !txtVerb.getText().equals("") && Integer.parseInt(txtVerb.getText()) <= 100 &&
								Integer.parseInt(txtVerb.getText()) >= 0 ) {
							limits.put("VERB", Double.parseDouble(txtVerb.getText())/100);
						} else {
							res = false;
						}							
					}
					
					if( res ) {
						JOptionPane.showMessageDialog(component, "Salvo com sucesso!");
						annotation.modifyAnnotationAllowed();
						dispose();
					} else
						JOptionPane.showMessageDialog(component, " Erro na escrita dos limites ");
					
				}
			});

	        cancel_button.setText("Cancelar");
	        cancel_button.addActionListener(new ActionListener() {
				
				@Override
				public void actionPerformed(ActionEvent e) {
					setVisible(false);
				}
			});
	        

	        verb_panel.setBorder(javax.swing.BorderFactory.createTitledBorder("Verbos"));

	        checkVerb.setText("Anotar todos os verbos");
	        checkVerb.setSelected(true);
	        checkVerb.addActionListener(new java.awt.event.ActionListener() {

				@Override
				public void actionPerformed(ActionEvent arg0) {
					if ( !checkVerb.isSelected() )
						txtVerb.setEnabled(true);
					else
						txtVerb.setEnabled(false);
					
				}
	        });

	        jLabel1.setText("Porcentagem da anotação (%): ");

	        javax.swing.GroupLayout verb_panelLayout = new javax.swing.GroupLayout(verb_panel);
	        verb_panel.setLayout(verb_panelLayout);
	        verb_panelLayout.setHorizontalGroup(
	            verb_panelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
	            .addGroup(verb_panelLayout.createSequentialGroup()
	                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
	                .addGroup(verb_panelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
	                    .addComponent(checkVerb, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
	                    .addComponent(jLabel1, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
	                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
	                .addComponent(txtVerb, javax.swing.GroupLayout.PREFERRED_SIZE, 33, javax.swing.GroupLayout.PREFERRED_SIZE))
	        );
	        verb_panelLayout.setVerticalGroup(
	            verb_panelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
	            .addGroup(verb_panelLayout.createSequentialGroup()
	                .addComponent(checkVerb)
	                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
	                .addGroup(verb_panelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
	                    .addComponent(jLabel1)
	                    .addComponent(txtVerb, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)))
	        );

	        noun_panel.setBorder(javax.swing.BorderFactory.createTitledBorder("Substantivo"));

	        checkNoun.setText("Anotar todos os substantivos");
	        checkNoun.setSelected(true);
	        checkNoun.addActionListener(new ActionListener() {
				
				@Override
				public void actionPerformed(ActionEvent e) {
					if( !checkNoun.isSelected() )
						txtNoun.setEnabled(true);
					else
						txtNoun.setEnabled(false);
				}
			});

	        jLabel2.setText("Porcentagem da anotação (%):");

	        txtNoun.setPreferredSize(new java.awt.Dimension(6, 20));
	        txtNoun.setEnabled(false);
	        txtVerb.setEnabled(false);
	        javax.swing.GroupLayout noun_panelLayout = new javax.swing.GroupLayout(noun_panel);
	        noun_panel.setLayout(noun_panelLayout);
	        noun_panelLayout.setHorizontalGroup(
	            noun_panelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
	            .addGroup(noun_panelLayout.createSequentialGroup()
	                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
	                .addGroup(noun_panelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
	                    .addComponent(checkNoun)
	                    .addGroup(noun_panelLayout.createSequentialGroup()
	                        .addComponent(jLabel2)
	                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
	                        .addComponent(txtNoun, javax.swing.GroupLayout.PREFERRED_SIZE, 33, javax.swing.GroupLayout.PREFERRED_SIZE))))
	        );
	        noun_panelLayout.setVerticalGroup(
	            noun_panelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
	            .addGroup(noun_panelLayout.createSequentialGroup()
	                .addComponent(checkNoun)
	                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
	                .addGroup(noun_panelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
	                    .addComponent(jLabel2)
	                    .addComponent(txtNoun, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)))
	        );

	        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
	        getContentPane().setLayout(layout);
	        layout.setHorizontalGroup(
	            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
	            .addGroup(layout.createSequentialGroup()
	                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
	                    .addGroup(layout.createSequentialGroup()
	                        .addContainerGap()
	                        .addComponent(verb_panel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
	                        .addGap(18, 18, 18)
	                        .addComponent(noun_panel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
	                        .addGap(0, 0, Short.MAX_VALUE))
	                    .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
	                        .addGap(0, 0, Short.MAX_VALUE)
	                        .addComponent(accept_button)
	                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
	                        .addComponent(cancel_button)))
	                .addContainerGap())
	        );
	        layout.setVerticalGroup(
	            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
	            .addGroup(layout.createSequentialGroup()
	                .addGap(24, 24, 24)
	                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
	                    .addComponent(noun_panel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
	                    .addComponent(verb_panel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
	                .addGap(18, 18, 18)
	                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
	                    .addComponent(cancel_button)
	                    .addComponent(accept_button))
	                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
	        );

	        pack();
	    }
	
	 /**
	  * This method obtains the limits of words to be annotated for verbs and nouns
	  * @return
	  */
	 public Map<String,Double> getLimits() {
		return limits;
		 
	 }
	 
	 /**
	  * This method set the limit of words to be annotated by an identifier (verb or noun)
	  * @param id can be VERB or NOUN
	  * @param value the limit of words
	  */
	 public void setLimits( String id, double value ) {
		 limits.put(id, value);
		 
		 if( id.equals("VERB") )
			 if( limits.get( id ) == 1.0 ) {
				 checkVerb.setSelected( true );
				 txtVerb.setEnabled( false );
				 txtVerb.setText( "" );
			 } else {
				 checkVerb.setSelected( false );
				 txtVerb.setEnabled( true );
				 txtVerb.setText("" + value*100 );
			 }
		 
		 if( id.equals("NOUN") )
			 if( limits.get( id ) == 1.0 ) {
				 checkNoun.setSelected( true );
				 txtNoun.setEnabled( false );
				 txtNoun.setText( "" );
			 } else {
				 checkNoun.setSelected( false );
				 txtNoun.setEnabled( true );
				 txtNoun.setText( "" + value*100 );
			 }		 
	 }
	 
	/**
	 * This method add the annotation to the view, for handling events 
	 * of the annotation
	 * @param annotation 
	 */
	public void setController(final Annotation annotation) {
		this.annotation = annotation;
	}

}
