package view;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.ScrollPaneConstants;
import javax.swing.SwingConstants;
import javax.swing.border.TitledBorder;

import model.ComplexPredicate;
import model.Language;
import model.MorphoTags;
import model.StateAnnotation;
import model.WordPC;
import model.wsd.graphwsd.GraphWSD;
import nlputil.Lemmar;
import view.components.Paragraph;
import view.components.Text;
import view.components.Tokens;
import controller.Annotation;
import controller.ComplexPredicateController;

/**
 * This class represent the component that show the text that will be anoted. In
 * this component, be shown tabs that correspond to texts loaded
 * 
 * @author fernando
 * 
 */
public class TextView extends JPanel {
	private Annotation annotation;

	private JPanel file_panel[];
	private JTabbedPane tabbed_panel;
	private JLabel wait_label;
	private JPanel typeAnnotate_panel;
	private JCheckBox checkSubstantivo;
	private JCheckBox checkVerbo;

	private Text[] texts;
	private File file[];

	// used to verify same words
	private HashMap<String, Node> list_words;

	// used to count the number of words with meaning
	private int meaning_words_nouns;
	private int meaning_words_verbs;

	private ComplexPredicateController cpController;

	private GraphWSD wsd;

	/**
	 * Constructor of class TextView, be created and added the components of GUI
	 * and init some variables of the instance
	 */
	public TextView() {
		super();

		setBorder(new TitledBorder("Visualizador de texto"));
		setLayout(new BorderLayout());

		typeAnnotate_panel = new JPanel(new FlowLayout(FlowLayout.LEFT));
		typeAnnotate_panel.setBorder(new TitledBorder("Classes Gramaticais"));
		checkSubstantivo = new JCheckBox("Substantivo");
		checkSubstantivo
				.setToolTipText("Clique para ativar ou desativar a visualização dos substantivos a serem etiquetados");
		checkSubstantivo.setSelected(false);
		checkSubstantivo.setEnabled(false);
		checkSubstantivo.setName("N");
		typeAnnotate_panel.add(checkSubstantivo);
		checkSubstantivo.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent evt) {
				modifiedStateAnnotationTokens(checkSubstantivo);

			}
		});

		checkVerbo = new JCheckBox("Verbo");
		checkVerbo
				.setToolTipText("Clique para ativar ou desativar a visualização dos verbos a serem etiquetados");
		checkVerbo.setEnabled(false);
		checkVerbo.setSelected(true);
		checkVerbo.setName("VERB");
		typeAnnotate_panel.add(checkVerbo);
		checkVerbo.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent evt) {
				modifiedStateAnnotationTokens(checkVerbo);
			}
		});
		add(typeAnnotate_panel, BorderLayout.NORTH);

		tabbed_panel = new JTabbedPane(SwingConstants.TOP,
				JTabbedPane.WRAP_TAB_LAYOUT);
		tabbed_panel.add(new JPanel());
		add(tabbed_panel, BorderLayout.CENTER);

		wait_label = new JLabel(
				"<html>Carregando os textos<br>Aguarde...</html>");
		wait_label.setIcon(new ImageIcon("img" + File.separator + "wait.gif"));
		wait_label.setVisible(false);
		add(wait_label, BorderLayout.SOUTH);

		list_words = new HashMap<String, Node>();

		texts = null;

		this.wsd = new GraphWSD();

		this.cpController = ComplexPredicateController.getInstance();
	}

	/**
	 * This method obtains the check component of verb
	 * 
	 * @return
	 */
	public JCheckBox getCheckVerbo() {
		return checkVerbo;
	}

	/**
	 * This method obtains the check component of noun
	 * 
	 * @return
	 */
	public JCheckBox getCheckSubstantivo() {
		return checkSubstantivo;
	}

	/**
	 * This method changes the tokens that must be showed as able to annotate
	 * 
	 * @param checkBox
	 *            who must be annotated (verb, noun or both)
	 */
	private void modifiedStateAnnotationTokens(JCheckBox checkBox) {
		try {
			for (int i = 0; i < texts.length; i++) {
				String cp = "";
				for (Tokens token : texts[i].getTokens()) {
					if (token != null) {
						if (checkBox.getName().equals("VERB")) {
							String vTag = token.getTag();
							if (token.getTag().equals("V"))
								vTag = "VERB";
							if (checkBox.getName().equals(vTag)) {
								cp = token.isPartOfComplexPredicate();
								if (!checkBox.isSelected()) {
									token.desactiveBorder();
								} else {
									token.activeBorder();
								}
							} else {
								if (token.getPartOfComplexPredicate()
										.equals(cp) && !cp.trim().equals("")) {
									if (checkBox.isSelected())
										token.activeBorderCP();
									else
										token.desactiveBorderCP();
								} else {
									token.desactiveBorderCP();
									cp = "";
								}
							}
						} else {
							if (checkBox.getName().equals(token.getTag())) {
								if (!checkBox.isSelected()) {
									token.desactiveBorder();
								} else {
									token.activeBorder();
								}
							}
						}

					}
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		paintComponents(getGraphics());
		repaint();
	}

	/**
	 * Enable all checkbox components
	 */
	public void enabledAnnotateTypes() {
		checkSubstantivo.setEnabled(true);
		checkSubstantivo.setSelected(false);
		checkVerbo.setEnabled(true);
		checkVerbo.setSelected(true);
	}

	/**
	 * This method cleans all to load a new set of text and a new annotation
	 */
	private void clear() {
		meaning_words_nouns = 0;
		meaning_words_verbs = 0;
		list_words.clear();

		texts = null;
		file_panel = null;

		tabbed_panel.removeAll();
	}

	/**
	 * Method to load the text ou collection of texts that will be annoted. In
	 * this method is loaded a formatted file according a pattern selected by
	 * MXPOST, i.e. the tokens are separated by space.
	 * 
	 * @param file
	 *            files to be loaded
	 * @throws IOException
	 *             It's thrown if some problem occur when it opens
	 */
	public void setText(File file[]) throws IOException {

		clear();

		this.file = file;
		wait_label.setVisible(true);

		openFile();

		wait_label.setVisible(false);

	}

	/**
	 * This method is used to init the open process of new files to be
	 * annotated.
	 * 
	 * In this method is used a file filter, this allows just the open of txt
	 * files
	 * 
	 * @throws IOException
	 *             if some problem in openning of files
	 */
	private void openFile() throws IOException {

		file_panel = new JPanel[file.length];
		texts = new Text[file.length];

		long total_time = 0;
		long time0 = System.nanoTime();
		System.out.println("Loading...");

		for (int i = 0; i < file.length; i++) {

			// file Format: UTF-8
			BufferedReader reader = new BufferedReader(new InputStreamReader(
					new FileInputStream(file[i]), "UTF-8")); // UTF-8

			texts[i] = new Text(file[i].getName(), file[i].getParentFile());

			// Layout
			file_panel[i] = new JPanel();
			file_panel[i].setOpaque(true);
			file_panel[i].setBackground(Color.white);
			file_panel[i].setLayout(new BoxLayout(file_panel[i],
					BoxLayout.Y_AXIS));

			// Adding a new tab to the panel
			tabbed_panel.addTab("Texto " + (i + 1), new JScrollPane(
					file_panel[i], ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED,
					ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED));
			tabbed_panel.setToolTipTextAt(i,
					file[i].getName().split(".mxpost")[0]);

			HashMap<String, Tokens> linked_words = new HashMap<String, Tokens>();

			String line = "";

			while ((line = reader.readLine()) != null) {

				System.out.println(line);

				tabbed_panel.repaint();

				// This verifications \n e \r\n are necessaries to not restrict
				// the use of windows or linux
				if (line.compareTo("") == 0 || line.compareTo("\r\n") == 0
						|| line.compareTo("\n") == 0)
					continue;

				String tokens[] = line.split(" ");
				Paragraph p = new Paragraph();
				try {
					for (int j = 0; j < tokens.length; j++) {

						System.out.println("\t" + tokens[j]);

						if (tokens[j].compareTo("") == 0
								|| tokens[j].compareTo("\r\n") == 0
								|| line.compareTo("\n") == 0)
							continue;

						/*
						 * Separating a word from his tag in the net format:
						 * treetagger = [ word, tag, lemma ] mxpost = [word,
						 * tag]
						 */

						String w[] = tokens[j].split("_");

						MorphoTags tag;
						if (w.length == 2)
							tag = MorphoTags.mxpostMap(w[1]);
						else
							tag = MorphoTags.treetaggerMap(w[1]);

						// Verifying the state annotation of word by the tag
						StateAnnotation state_annotation;
						if (tag == MorphoTags.NOUN) {
							state_annotation = StateAnnotation.NOUN_NO_ANNOTATED;
						} else if (tag == MorphoTags.VERB) {
							state_annotation = StateAnnotation.VERB_NO_ANNOTATED;
						} else if (tag == MorphoTags.VAUX) {
							state_annotation = StateAnnotation.VERB_AUX_NO_ANNOTATED;
						} else {
							if (tag != MorphoTags.OTHER) {
								state_annotation = StateAnnotation.NO_ANNOTATED;
							} else {
								state_annotation = StateAnnotation.NO_ANNOTE;
							}
						}

						// Creating the tokens that will be showed in the screen
						Tokens n = new Tokens(w[0], state_annotation, w[1],
								tag, texts[i].getLanguage());
						n.setMorphoTag(tag);

						// Adding a token to the token list
						texts[i].addToken(n);

						// Lemmatizing the token
						if (n != null) {
							if (n.getAnnotedState() != StateAnnotation.NO_ANNOTE) {

								if (!w[0].isEmpty() && w[0].length() >= 1) {
									String lemma = n.getText();
									if (n.getLanguage() == Language.PORTUGUESE) {

										if (Character.isLetter(lemma.charAt(0))) {
											long l_time0 = System.nanoTime();
											String tTag = n.getTag();
											if (n.getTag().equals("V"))
												tTag = "VERB";
											n.setLemma(Lemmar
													.lemma(lemma, tTag));
											long l_time1 = System.nanoTime();
											total_time += (l_time1 - l_time0);
										}

									} else {
										// Using TreeTagger
										n.setLemma(w[2]);
									}
								}

								if (linked_words.containsKey(n.getLemma() + "_"
										+ n.getMorphoTag())) {
									Tokens t = linked_words.get(n.getLemma()
											+ "_" + n.getMorphoTag());
									t.addLink(n);
									n.addLink(t);
								} else {
									linked_words.put(
											n.getLemma() + "_"
													+ n.getMorphoTag(), n);
								}
							}
						}

						p.add(n);
					}
				} catch (Exception e) {
					errorPrint("Houve um erro ao carregar o arquivo de texto "
							+ texts[i].getNameFile());
					e.printStackTrace();
				}
				texts[i].addToken(null);
				file_panel[i].add(p);
			}
			reader.close();
		}

		enabledAnnotateTypes();
		// Adding mouse events to the selectable tokens
		Tokens n;
		for (int i = 0; i < texts.length; i++) {
			LinkedList<Tokens> list = texts[i].getTokens();
			for (int j = 0; j < list.size(); j++) {
				n = list.get(j);
				if (n != null)
					if (n.getAnnotedState() != StateAnnotation.NO_ANNOTE)
						addEvent(n);
			}
		}

		setComplexPredicates();

		/*
		 * Print of time log
		 */
		long time1 = System.nanoTime();
		System.out.println("Tempo(s): " + (time1 - time0) / 1000000000);
		System.out.println("Tempo(ns): " + (time1 - time0));

		System.out.println("\nLematizacao");
		System.out.println("Tempo(s): " + (total_time / 1000000000));
		System.out.println("Tempo(ns): " + total_time);

		System.out.println("\nFrequencia");
		time0 = System.nanoTime();
		makeFrequency();
		time1 = System.nanoTime();
		System.out.println("Tempo(s): " + (time1 - time0) / 1000000000);
		System.out.println("Tempo(ns): " + (time1 - time0));

		modifyCheckboxs();

		wait_label.setVisible(false);
		paintComponents(getGraphics());
		repaint();

	}

	/**
	 * This method shows the error message in the UI
	 * 
	 * @param message
	 *            message to showed
	 */
	private void errorPrint(String message) {

		JOptionPane.showMessageDialog(this, message, "Erro",
				JOptionPane.ERROR_MESSAGE);
	}

	/**
	 * This method modifies the state of checkbox
	 */
	public void modifyCheckboxs() {
		modifiedStateAnnotationTokens(checkSubstantivo);
		modifiedStateAnnotationTokens(checkVerbo);
	}

	/**
	 * This method analize a token (verb) and return if this can be part of a
	 * complex predicate
	 * 
	 * @param n
	 *            the toke to analize if it is part of a complex predicate
	 * @return
	 */
	private ComplexPredicate verbOfComplexPredicate(Tokens n) {
		String lemma = n.getLemma();
		if (cpController.get_complex_predicates().containsKey(lemma))
			return cpController.get_complex_predicates().get(lemma);
		return null;
	}

	/**
	 * This method analize all of tokens in a text loaded and set the complex
	 * predicates
	 */
	private void setComplexPredicates() {
		try {
			for (int i = 0; i < texts.length; i++) {
				List<Integer> indexCP = new ArrayList<Integer>();
				WordPC word = null;
				int j = 0;
				for (Tokens token : texts[i].getTokens()) {
					if (token != null) {
						if (token.getMorphoTagPOS().equals(MorphoTags.VERB)) {
							ComplexPredicate cp = verbOfComplexPredicate(token);
							if (cp != null) {
								indexCP.clear();
								indexCP.add(j);
								word = cp.getRoot();
							}
						} else {
							if (indexCP.size() > 0) {
								word = word.getChild(token.getLemma());
								if (word == null)
									indexCP.clear();
								else {
									indexCP.add(j);
									if (word.getChildren().size() == 0) {
										for (Integer index : indexCP) {
											if (word.getType().equals("i"))
												texts[i].getTokens()
														.get(index.intValue())
														.setPartOfComplexPredicate(
																"CPI");
											else
												texts[i].getTokens()
														.get(index.intValue())
														.setPartOfComplexPredicate(
																"CPNI");
										}
										indexCP.clear();
										word = null;
									}

								}
							}
						}
					}
					j++;
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * Generate a file with frequencies for every word in the cluster
	 */
	public void makeFrequency() {
		ArrayList<Node> frequency_verb_list = new ArrayList<Node>();
		ArrayList<Node> frequency_noun_list = new ArrayList<Node>();
		Node n;
		Map<String, Double> limits = null;
		try {
			limits = annotation.getLimitsAnnotation();

		} catch (Exception e) {
			e.printStackTrace();
		}

		try {
			Iterator<Entry<String, Node>> iterator = list_words.entrySet()
					.iterator();

			while (iterator.hasNext()) {
				n = iterator.next().getValue();
				if (n.t.getMorphoTag() == MorphoTags.NOUN)
					frequency_noun_list.add(n);
				else if (n.t.getMorphoTag() == MorphoTags.VERB)
					frequency_verb_list.add(n);
			}

			Collections.sort(frequency_noun_list, new Comparator<Node>() {
				@Override
				public int compare(Node n1, Node n2) {
					if (n1.frequency > n2.frequency)
						return -1;
					if (n1.frequency < n2.frequency)
						return 1;
					return 0;
				}
			});

			Collections.sort(frequency_verb_list, new Comparator<Node>() {
				@Override
				public int compare(Node n1, Node n2) {
					if (n1.frequency > n2.frequency)
						return -1;
					if (n1.frequency < n2.frequency)
						return 1;
					return 0;
				}
			});

		} catch (Exception e) {
			e.printStackTrace();
		}
		BufferedWriter writer;
		try {
			if (texts == null)
				return;
			writer = new BufferedWriter(
					new OutputStreamWriter(
							new FileOutputStream(texts[0].getDirectory()
									.getAbsolutePath()
									+ File.separator
									+ texts[0].getDirectory().getName()
									+ ".frequencia"), "UTF-8"));
			writer.write("Nro de substantivos = " + meaning_words_nouns);

			writer.newLine();
			writer.newLine();
			writer.write("Palavra\tFrequência");
			writer.newLine();

			int i = 0;
			int frequency_value_nouns = (int) (this.meaning_words_nouns * limits
					.get("NOUN").doubleValue());
			for (i = 0; i < frequency_noun_list.size(); i++) {

				int count_noun = frequency_noun_list.get(i).frequency;
				while (count_noun > 0 && frequency_value_nouns > 0) {

					frequency_value_nouns--;
					count_noun--;
				}
				if (frequency_value_nouns == 0) {
					frequency_value_nouns = frequency_noun_list.get(i).frequency;
					break;
				}
			}
			i = 0;
			for (; i < frequency_noun_list.size(); i++) {

				n = frequency_noun_list.get(i);
				if (n.frequency < frequency_value_nouns) {
					break;
				}
				writer.write(n.t.getLemma() + "\t" + n.frequency);
				writer.newLine();
				if (!n.t.getAnnotedState().equals(StateAnnotation.ANNOTATED)
						&& !n.t.getAnnotedState().equals(
								StateAnnotation.PREV_ANNOTATED)) {
					n.t.setType(StateAnnotation.NOUN_NO_ANNOTATED);
					ArrayList<Tokens> aux = n.t.getLinkedTokens();
					for (int j = 0; j < aux.size(); j++) {
						aux.get(j)
								.setType(StateAnnotation.NOUN_NO_ANNOTATED);
					}
				}
			}

			writer.write("-----------");
			writer.newLine();
			for (; i < frequency_noun_list.size(); i++) {

				n = frequency_noun_list.get(i);
				writer.write(n.t.getLemma() + "\t" + n.frequency);
				writer.newLine();
				if (!n.t.getAnnotedState().equals(StateAnnotation.ANNOTATED)
						&& !n.t.getAnnotedState().equals(
								StateAnnotation.PREV_ANNOTATED)) {
					n.t.setType(StateAnnotation.NO_ANNOTATED);
					n.t.activeBorder();
					ArrayList<Tokens> aux = n.t.getLinkedTokens();
					for (int j = 0; j < aux.size(); j++) {
						aux.get(j)
								.setType(StateAnnotation.NO_ANNOTATED);
						aux.get(j).activeBorder();
					}
				}
			}

			writer.newLine();
			writer.write("Nro de verbos = " + meaning_words_verbs);
			writer.newLine();
			int frequency_value_verbs = (int) (this.meaning_words_verbs * limits
					.get("VERB").doubleValue());
			for (i = 0; i < frequency_verb_list.size(); i++) {
				int count_verb = frequency_verb_list.get(i).frequency;
				while (count_verb > 0 && frequency_value_verbs > 0) {

					frequency_value_verbs--;
					count_verb--;
				}
				if (frequency_value_verbs == 0) {
					frequency_value_verbs = frequency_verb_list.get(i).frequency;
					break;
				}
			}

			i = 0;
			for (; i < frequency_verb_list.size(); i++) {

				n = frequency_verb_list.get(i);

				if (n.frequency < frequency_value_verbs) {
					break;
				}
				writer.write(n.t.getLemma() + "\t" + n.frequency);
				writer.newLine();

				if (!n.t.getAnnotedState().equals(StateAnnotation.ANNOTATED)
						&& !n.t.getAnnotedState().equals(
								StateAnnotation.PREV_ANNOTATED)) {
					n.t.setType(StateAnnotation.VERB_NO_ANNOTATED);
					ArrayList<Tokens> aux = n.t.getLinkedTokens();
					for (int j = 0; j < aux.size(); j++) {
						aux.get(j)
								.setType(StateAnnotation.VERB_NO_ANNOTATED);
					}
				}
			}

			writer.write("-----------");
			writer.newLine();
			for (; i < frequency_verb_list.size(); i++) {

				n = frequency_verb_list.get(i);
				writer.write(n.t.getLemma() + "\t" + n.frequency);
				writer.newLine();

				if (!n.t.getAnnotedState().equals(StateAnnotation.ANNOTATED)
						&& !n.t.getAnnotedState().equals(
								StateAnnotation.PREV_ANNOTATED)) {
					n.t.setType(StateAnnotation.NO_ANNOTATED);
					n.t.activeBorder();
					ArrayList<Tokens> aux = n.t.getLinkedTokens();
					for (int j = 0; j < aux.size(); j++) {
						aux.get(j)
								.setType(StateAnnotation.NO_ANNOTATED);
						aux.get(j).activeBorder();
					}
				}
			}

			writer.close();

		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	/**
	 * This method adds events to a token
	 * 
	 * @param n
	 *            token that will be modified
	 */
	private void addEvent(Tokens n) {
		// Verifica se a palavra já existia na lista de palavras
		Node equal_world = list_words.get(n.getLemma().trim() + "_"
				+ n.getMorphoTag());
		// Se já existe, adiciona na palavra atual um link para a palavra ja
		// existente
		if (equal_world != null) {

			if (n.getMorphoTag() == MorphoTags.NOUN) {
				equal_world.frequency++;
				meaning_words_nouns++;
				equal_world.t.addLink(n);
				n.addLink(equal_world.t);
			}

			if (n.getMorphoTag() == MorphoTags.VERB) {
				equal_world.frequency++;
				meaning_words_verbs++;
				equal_world.t.addLink(n);
				n.addLink(equal_world.t);
			}

		} else {
			if (n.getMorphoTag() == MorphoTags.NOUN) {
				meaning_words_nouns++;
				list_words.put(n.getLemma().trim() + "_" + n.getMorphoTag(),
						new Node(n));
			}

			if (n.getMorphoTag() == MorphoTags.VERB) {
				meaning_words_verbs++;
				list_words.put(n.getLemma().trim() + "_" + n.getMorphoTag(),
						new Node(n));
			}

		}

		final Component component = n;
		n.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseClicked(final MouseEvent e) {
				annotation.translate((Tokens) e.getSource());

				if (e.isMetaDown()) {
					JPopupMenu popMenu = new JPopupMenu();
					if (checkSubstantivo.isSelected()) {
						JMenuItem jMenuSubstantivo = new JMenuItem(
								"Substantivo");
						jMenuSubstantivo
								.addActionListener(new ActionListener() {

									@Override
									public void actionPerformed(ActionEvent arg0) {
										annotation
												.setMorphoTagtoToken(MorphoTags.NOUN);
										annotation
												.changeType(StateAnnotation.NOUN_NO_ANNOTATED);
										annotation.setTag("N");
									}
								});
						popMenu.add(jMenuSubstantivo);
					}
					if (checkVerbo.isSelected()) {
						JMenuItem jMenuVerbo = new JMenuItem("Verbo");
						jMenuVerbo.addActionListener(new ActionListener() {

							@Override
							public void actionPerformed(ActionEvent e) {
								annotation.setMorphoTagtoToken(MorphoTags.VERB);
								annotation
										.changeType(StateAnnotation.VERB_NO_ANNOTATED);
								annotation.setTag("VERB");
							}
						});
						popMenu.add(jMenuVerbo);
					}
					popMenu.show(component, 11, 11);
				}
				annotation.enableComment();
			}
		});
	}

	/**
	 * This method set the text into the text components
	 * 
	 * @param text
	 *            texts to be set into the component
	 */
	public void setText(Text text[]) {

		clear();

		this.texts = text;
		file_panel = new JPanel[texts.length];

		// Adicionando os componentes visuais
		for (int i = 0; i < texts.length; i++) {

			file_panel[i] = new JPanel();
			file_panel[i].setLayout(new BoxLayout(file_panel[i],
					BoxLayout.PAGE_AXIS));

			Paragraph p = new Paragraph();
			for (Tokens token : texts[i].getTokens()) {

				if (token != null)
					if (token.getAnnotedState() != StateAnnotation.NO_ANNOTE)
						addEvent(token);

				if (token == null) {
					file_panel[i].add(p);
					p = new Paragraph();
				} else
					p.add(token);
			}

			tabbed_panel.addTab("Texto " + (i + 1), new JScrollPane(
					file_panel[i], ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED,
					ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED));
			tabbed_panel.setToolTipTextAt(i, texts[i].getNameFile());
		}

		modifyCheckboxs();
	}

	/**
	 * This method add the annotation to the view, for handling events of the
	 * annotation
	 * 
	 * @param annotation
	 */
	public void setController(Annotation annotation) {
		this.annotation = annotation;
	}

	public Text[] getTexts() {
		return texts;
	}

	private class Node {
		protected Tokens t;
		protected int frequency;

		public Node(Tokens t) {
			this.t = t;
			this.frequency = 1;
		}
	}
}
