package view.components;

import java.awt.Color;
import java.awt.Component;

import javax.swing.BorderFactory;
import javax.swing.DefaultListCellRenderer;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.ListCellRenderer;

import edu.smu.tspell.wordnet.NounSynset;
import edu.smu.tspell.wordnet.Synset;
import edu.smu.tspell.wordnet.SynsetType;
import edu.smu.tspell.wordnet.impl.file.synset.VerbReferenceSynset;

/**
 * This class is responsible for handling the synset format 
 * @author fernando
 *
 */
public class SynsetCellRender  implements ListCellRenderer{
	protected DefaultListCellRenderer defaultRenderer = new DefaultListCellRenderer();
	private Synset synset;
	
	private boolean viewHypernyms;
	private boolean viewtHyponyms;
	
	@Override
	public Component getListCellRendererComponent(JList list, Object value,
			int index, boolean isSelected, boolean cellHasFocus) {
		
		JLabel renderer = (JLabel) defaultRenderer
		        .getListCellRendererComponent(list, value, index, isSelected,
		                true );
		
		renderer.setBorder( BorderFactory.createCompoundBorder(
				BorderFactory.createEmptyBorder(5, 5, 5, 5), 
				BorderFactory.createLineBorder( Color.GRAY, 1) )
				);
		
		this.synset = ((Synset) value);		
		// 11-10-2013
		Synset hypers[] = null;
		Synset hypo[]  = null;
		if( this.synset.getType() == SynsetType.NOUN ) {
			hypers = ((NounSynset) synset).getHypernyms();
			hypo = ((NounSynset) synset).getHyponyms();
		} else	if( this.synset.getType() == SynsetType.VERB ) {
			hypers = ((VerbReferenceSynset) synset).getHypernyms();
			hypo = ((VerbReferenceSynset) synset).getTroponyms();
		}
		
		String samples="";
		int count = 1;
		for(String aux : synset.getUsageExamples() ){
			samples+="<li>Ex. " + count + ": " + aux +"\n";
			count++;
		}
		if ( count > 1)
			samples = "<ul>" + samples + "</ul>";
		
		//Hiperônymous
		String hypeText = "";
		if ( viewHypernyms ){
			count = 1;
			hypeText = "<ul>";
			for( Synset h : hypers ){
				hypeText+="<li>Hiperônimo " + count + ":" + formatSynonyms( h ) + " " + h.getDefinition();
				count++;
			}
			hypeText += "</ul>";
		}
		
		//Hipônymous
		String hypoText = "";
		if ( viewtHyponyms ){
			count = 1;
			hypoText = "<ul>";
			for( Synset h : hypo ){
				hypoText+="<li>Hipônimo " + count + ":" + formatSynonyms( h ) + " " + h.getDefinition();
				count++;
			}
			hypoText += "</ul>";
		}
		
		renderer.setText("<html>" +
				(index + 1) + " -- " +
				formatSynonyms( synset ) + "<br />" +
				synset.getDefinition() + 
				samples+
				hypeText+
				hypoText+
				"</html>");
		
		return renderer;
	}
	
	/**
	 * This method provides a format to show the synset
	 * @param synset the synset to be formatted
	 * @return
	 */
	public static String formatSynonyms( Synset synset ){
		
		String text = "";
		boolean first = true;
		for(String aux : synset.getWordForms() ){
			if ( first ){
				text = aux;
				first = false;
				continue;
			}
			
			text+= ", " + aux;
		}
		
		return "{" + text + "}"; 
	}
	
	/**
	 * This method return the synset
	 * @return
	 */
	public Synset getSynset(){
		return synset;
	}

	/**
	 * This method set if must be showed the hyperonymous
	 * @param flag
	 */
	public void setViewHypernyms( boolean flag ){
		this.viewHypernyms = flag;
	}
	
	/**
	 * This method set if must be showed the hyponymous
	 * @param flag 
	 */
	public void setViewtHyponyms( boolean flag ){
		this.viewtHyponyms = flag;
	}
}
